/* 05.08.2001: automatic MCP1 update procedure with support for the        */
/*             following components: Warp 4 FI products, 3rd party add-ons */
/* 06.03.2001: added clean-up plugin                                       */
/* 06.10.2001: bypassed problem with Quercus Systems Personal REXX         */
/* 06.13.2001: made sure that directory fix always exists                  */
/* 06.16.2001: fixed bug in deldir with relocated burn directory           */
/* 07.15.2001: changed execute not to generate warning on delete           */
/* 08.12.2001: added support for CP1 fixpak                                */

/* read command line parameters */
parse arg cddrive

/* load configuration file */
cfgfile = 'updcd.cfg'
do while lines(cfgfile)
	l = linein(cfgfile)
	interpret l
end
rc = lineout(cfgfile)

/* check cddrive value */
if substr(cddrive, 2) = ':' then w4cd = translate(cddrive)

/* make sure they exist */
'@mkdir 'burn' >nul 2>>&1'
'@mkdir 'fixes' >nul 2>>&1'
'@mkdir 'updates' >nul 2>>&1'
'@mkdir fix >nul 2>>&1'

/* need to know absolute path */
cdir  	= directory()
burn 		= directory(burn)
cdir  	= directory(cdir)
fixes 	= directory(fixes)
cdir  	= directory(cdir)
updates = directory(updates)
cdir  	= directory(cdir)

/* log file */
if filespec('drive', log) = '' then log	= strip(cdir, 'T', '\')||'\'||log

/* updcd version */
ver = 1.7
n.errors = 0
n.warnings = 0
n.elapsed = time('E')

/* load rexxutil functions we need */
call rxfuncadd sysfiletree, rexxutil, sysfiletree
call rxfuncadd syscurstate, rexxutil, syscurstate

/* remember we were started */
'@echo go4it busy > go4it.bsy'

/* clean up things */
'@mode 80,25 >nul'
'@ansi on >NUL'
say '1B'x || '[36;44m'
say '1B'x || '[1m'
'@cls'
say
say ' Searching for components which can be integrated. Please wait!'
call syscurstate off

/* find fixpaks located in the subdirectory fixes */
say
startdir = fixes
fpk. = ''
rc = lineout(, " Fixpak:    Type:     Directory:")
rc = lineout(, " -------------------------------")
rc = SysFileTree(startdir'\srv_prod.*', ini., 'FSO') 
do i = 1 to ini.0
	/* say ini.i */
	do while lines(ini.i)
		l=linein(ini.i)

		if pos('Convenience Package', l) > 0 & fpk.cp1.fnd <> 1 then do
			fpk.cp1.fnd = 1
			fpk.cp1.typ = substr(l, pos('XR', l), 7)
			fpk.cp1.num = substr(fpk.cp1.typ, 5)
			fpk.cp1.dir = translate(substr(ini.i, 1, lastpos('\', ini.i)-1))
			fpk.cp1.dir = substr(fpk.cp1.dir, 1, lastpos('\FIX\', fpk.cp1.dir)-1)
			rc = lineout(, " "fpk.cp1.typ"    CP_1      "fpk.cp1.dir)
			fpk.cp1.typ = 'XR'substr(fpk.cp1.typ, 4)
			leave
		end

		if pos('IBM OS/2 Base Device Drivers', l) > 0 & fpk.dd.fnd <> 1 then do
			fpk.dd.fnd = 1
			fpk.dd.typ = substr(l, pos('XR', l), 7)
			fpk.dd.num = substr(fpk.dd.typ, 5)
			fpk.dd.dir = translate(substr(ini.i, 1, lastpos('\', ini.i)-1))
			fpk.dd.dir = substr(fpk.dd.dir, 1, lastpos('\FIX\', fpk.dd.dir)-1)
			rc = lineout(, " "fpk.dd.typ  "   Driver    "fpk.dd.dir)
			fpk.dd.typ = 'XR'substr(fpk.dd.typ, 4)
			leave
		end

	end
	rc = lineout(ini.i)
end
if ini.0 = 0 then call lineout "", " Found nothing..."

/* find updated products located in the subdirectory updates */
say
startdir = updates
upd. = ''
rc = lineout(, " Update:    Type:     Directory:")
rc = lineout(, " -------------------------------")
rc = SysFileTree(startdir'\syslevel.*', ini., 'FSO') 

do i = 1 to ini.0
	/* say ini.i */
	do while lines(ini.i)
		l=linein(ini.i)
	end
	rc = lineout(ini.i)
end
if ini.0 = 0 then call lineout "", " Found nothing..."

/* find addons and Warp 4 FI products */
say
if stream('addons\read.me', 'c', 'query exists') <> '' then do
			call sysfiletree 'addons\*', 'ini.', 'DO'
			rc = lineout(, ' Found 'ini.0-2' add-on products in 'directory()||'\ADDONS')
end
if stream('fi\fibase.rsp', 'c', 'query exists') <> '' then do
			call sysfiletree 'fi\*', 'ini.', 'DO'
			rc = lineout(, ' Found 'ini.0-1' Warp 4 Feature Installer products in 'directory()||'\FI')
end

/* determining language to use */
ini = w4cd'\os2image\disk_2\syslevel.os2'
if stream(ini, 'c', 'query exists') <> '' then do
	do while lines(ini)
			l=linein(ini)
			if pos('Convenience Package', l) > 0 then do
				lang = substr(l, pos('XR', l), 3)
				/* say lang */
			end
	end
	rc = lineout(ini)
end
else do
	Say
	Say ' No CD-ROM found! Exiting...'
	rc = lineout(log, 'Error: No CD-ROM found! Exiting...')
	'@del go4it.bsy >nul 2>>&1'
	exit 9
end

select
	when lang = 'XR0' then lang = 'EN (English)'
	when lang = 'XRG' then lang = 'DE (German)'
	when lang = 'XRN' then lang = 'NO (Norwegian)'
	when lang = 'XRV' then lang = 'TW (Thai)'
	when lang = 'XRF' then lang = 'FR (French)'
	when lang = 'XRI' then lang = 'IT (Italian)'
	when lang = 'XRJ' then lang = 'JP (Japanese)'
	when lang = 'XRK' then lang = 'KO (Korean)'
	when lang = 'XRB' then lang = 'BR (Brazilian)'
	when lang = 'XRC' then lang = 'CN (Chinese)'
	when lang = 'XRS' then lang = 'ES (Spanish)'
	when lang = 'XRD' then lang = 'DK (Danish)'
	when lang = 'XRL' then lang = 'FI (Finnish)'
	when lang = 'XRH' then lang = 'NL (Dutch)'
	when lang = 'XRY' then lang = 'HU (Hungarian)'
	when lang = 'XRR' then lang = 'RU (Russian)'
  when lang = 'XRA' then lang = 'AR (Arabic)'
	otherwise lang = 'Unknown (assuming English for universal products)'
end
rc = lineout(, " Detected language: "lang)

say
say ' Press any key to start the build process or CTRL-C to abort'
'@pause >nul'

/* switch off deldir if set */
ddir = value("DELDIR", , "OS2ENVIRONMENT")
if length(ddir) > 0 then do
	rc = lineout(log, 'Info: Switching off deldir...')
	ddir = value("DELDIR", "", "OS2ENVIRONMENT")
end

'@del 'log' >nul 2>>&1'
rc = lineout(log, 'Info: Searching for fixes and updates...')
rc = lineout(log, 'Info: Found fix 'fpk.cp1.typ' CP_1 'fpk.cp1.dir)
rc = lineout(log, 'Info: Found fix 'fpk.dd.typ' Driver 'fpk.dd.dir)
rc = lineout(log, 'Info: Detected language: 'lang)
rc = lineout(log)
parse var lang w1 w2
lang = space(w1)
if lang = 'HU' | lang = 'RU' | lang = 'AR' | lang = 'Unknown' then lang = 'EN'

/* check if we have at least the base fixes */
if fpk.dd.dir = '' | fpk.cp1.dir = '' then do
	Say 'At least the base CP fixes should be present! Exiting...'
	rc = lineout(log, 'Error: At least the base CP fixes should be present! Exiting...')
	'@del go4it.bsy >nul 2>>&1'
	exit 9
end

/**********************/
/* Clean up old image */
/**********************/

'@cls'
say '1B'x || '[2;2HUpdate OS/2 installation CD-ROM, version 'ver
say '1B'x || '[4;2HTarget directory      : 'burn
say '1B'x || '[5;2HSource directory      : 'burn
say '1B'x || '[7;2HCurrent activity      : 'substr('Cleaning up directory: 'burn, 1, 50)
say '1B'x || '[8;2HTarget object         : 'substr('Searching...', 1, 50)
say '1B'x || '[9;2HSource object         : 'substr('Searching...', 1, 50)
say '1B'x || '[11;2HUpdate action         : 'substr('Searching...', 1, 50)
say '1B'x || '[12;2HUpdate return code    : 0'
say '1B'x || '[14;2HElapsed time [m]      : 'trunc(time('E')/60 + n.elapsed, 2)
say '1B'x || '[15;2HNumber of errors      : 'n.errors
say '1B'x || '[16;2HNumber of warnings    : 'n.warnings
say '1B'x || '[17;2HUpdate log            : 'log
say '1B'x || '[19;2H'

/* clean up burn */
plugin = 'cleanup.pin'
do while lines(plugin)
	l = linein(plugin)
	interpret l
end
call lineout plugin

/* remove RO attributes */
call execute '@attrib -r -s -h -a * /s'
/* 
this should be enabled if fies or updates get supported 
call execute '@attrib -r -s -h -a 'fixes'\* /s'
call execute '@attrib -r -s -h -a 'updates'\* /s'
*/

/* create some directories */
'@mkdir 'burn '>nul 2>&1'
call execute '@mkdir 'burn'\updcd'
call execute '@mkdir 'burn'\updcd\bootimg'
call execute '@mkdir 'burn'\updcd\bootimg\DISK_0_1.IMG'
call execute '@mkdir 'burn'\updcd\bootimg\DISK_2.IMG'

/*************************************/
/* Update installation floppy images */
/*************************************/

say '1B'x || '[4;26H'substr(burn, 1, 50)
say '1B'x || '[5;26H'substr(w4cd, 1, 50)
say '1B'x || '[7;26H'substr('Copying CD-ROM files', 1, 50)
say '1B'x || '[14;26H'trunc(time('E')/60 + n.elapsed, 2)
call execute '@mkdir 'burn'\diskimgs'
call execute '@xcopy 'w4cd'\diskimgs\* 'burn'\diskimgs\. /H/O/T/R/V'
call execute '@mkdir 'burn'\diskimgs\os2'
call execute '@mkdir 'burn'\diskimgs\os2\35'
call execute '@copy 'w4cd'\diskimgs\os2\35\disk0.dsk 'burn'\diskimgs\os2\35\.'
call execute '@copy 'w4cd'\diskimgs\os2\35\disk1_cd.dsk 'burn'\diskimgs\os2\35\.'
call execute '@copy 'w4cd'\diskimgs\os2\35\disk2.dsk 'burn'\diskimgs\os2\35\.'

/* seems we need REXXSOM.DLL */
call execute '@unpack 'w4cd'\os2image\disk_4\rexx fix /N:OREXXSOM.DLL'
call execute '@copy fix\orexxsom.dll fix\rexxsom.dll'
call execute '@del fix\orexxsom.dll'

/* first copy the stuff we need to the fix directory */
say '1B'x || '[7;26H'substr('Copying base CP fixes', 1, 50)
say '1B'x || '[14;26H'trunc(time('E')/60 + n.elapsed, 2)
call execute '@xcopy 'fpk.dd.dir'\fix\bdd.1\cd_boot.fl_  fix\.'
call execute '@xcopy 'fpk.cp1.dir'\fix\os2.1\* fix\.'
call execute '@xcopy 'fpk.cp1.dir'\fix\os2.5\* fix\.'

/* then prepare it */
'@del result.out >nul 2>&1'
'@call prpfp fix 'n.errors n.warnings trunc(time('E')/60 + n.elapsed, 2) ver log
res = linein('result.out')
rc = lineout('result.out')
parse var res n.errors n.warnings

/* finally run update */
say '1B'x || '[7;26H'substr('Updating installation floppy images', 1, 50)
say '1B'x || '[14;26H'trunc(time('E')/60 + n.elapsed, 2)
'@del result.out >nul'
'@call updif 'burn' fix 'n.errors n.warnings trunc(time('E')/60 + n.elapsed, 2) log ver expert add_dani add_dummy upd.fd1.dir upd.fd2.dir 
res = linein('result.out')
rc = lineout('result.out')
parse var res n.errors n.warnings 

/* do the rest */
say '1B'x || '[4;26H'substr(burn, 1, 50)
say '1B'x || '[5;26H'substr(w4cd, 1, 50)
say '1B'x || '[7;26H'substr('Copying CD-ROM files', 1, 50)
say '1B'x || '[14;26H'trunc(time('E')/60 + n.elapsed, 2)

call execute '@xcopy 'w4cd'\* 'burn'\. /H/O/T/R/V'

call execute '@mkdir 'burn'\cid'
call execute '@xcopy 'w4cd'\cid\* 'burn'\cid\. /H/O/T/S/R/V/E'

call execute '@mkdir 'burn'\ibminst'
call execute '@xcopy 'w4cd'\ibminst\* 'burn'\ibminst\. /H/O/T/S/E/R/V'

call execute '@mkdir 'burn'\info'
call execute '@xcopy 'w4cd'\info\* 'burn'\info\. /H/O/T/S/E/R/V'

call execute '@mkdir 'burn'\options'
call execute '@xcopy 'w4cd'\options\* 'burn'\options\. /H/O/T/S/E/R/V'

call execute '@mkdir 'burn'\report'
call execute '@xcopy 'w4cd'\report\* 'burn'\report\. /H/O/T/S/E/R/V'

call execute '@mkdir 'burn'\books'
call execute '@xcopy 'w4cd'\books\* 'burn'\books\. /H/O/T/S/E/R/V'

call execute '@mkdir 'burn'\os2image'
call execute '@xcopy 'w4cd'\os2image\* 'burn'\os2image\. /H/O/T/S/R/V/E'

/* remove RO attribute */
rc = SysFileTree(burn'\*', ini., 'BSO',,'**---')  
call execute '@attrib -r -s -h -a 'burn'\* /s'

/***********************************/
/* add Warp 4 FI products to updcd */
/***********************************/
if stream('fi\fibase.rsp', 'c', 'query exists') <> '' then do
	call execute '@mkdir 'burn'\fi'
	call execute '@xcopy fi\* 'burn'\fi\. /H/O/T/S/R/V/E'
	if stream(burn'\fi\coaches\os2\cchmain.exe', 'c', 'query exists') <> '' then do
		call execute '@copy fix\cchmain.exe 'burn'\fi\coaches\os2\.'
	end
	call execute '@del  fix\cchmain.exe'
	if stream(burn'\fi\coaches\os2\instprt.exe', 'c', 'query exists') <> '' then do
		call execute '@copy fix\instprt.exe 'burn'\fi\coaches\os2\.'
	end
	call execute '@del  fix\instprt.exe'
	if stream(burn'\fi\coaches\os2\dll\cchuser.dll', 'c', 'query exists') <> '' then do
		call execute '@copy fix\cchuser.dll 'burn'\fi\coaches\os2\dll\.'
	end
	call execute '@del  fix\cchuser.dll'
	if stream(burn'\fi\coaches\os2\dll\drvondsk.dll', 'c', 'query exists') <> '' then do
		call execute '@copy fix\drvondsk.dll 'burn'\fi\coaches\os2\dll\.'
	end
	call execute '@del  fix\drvondsk.dll'
	if stream(burn'\fi\coaches\os2\dll\portplug.dll', 'c', 'query exists') <> '' then do
		call execute '@copy fix\portplug.dll 'burn'\fi\coaches\os2\dll\.'
	end
	call execute '@del  fix\portplug.dll'
	if stream(burn'\fi\coaches\os2\dll\prtrlist.dll', 'c', 'query exists') <> '' then do
		call execute '@copy fix\prtrlist.dll 'burn'\fi\coaches\os2\dll\.'
	end
	call execute '@del  fix\prtrlist.dll'
	if stream(burn'\fi\coaches\os2\dll\sgmlpars.dll', 'c', 'query exists') <> '' then do
		call execute '@copy fix\sgmlpars.dll 'burn'\fi\coaches\os2\dll\.'
	end
	call execute '@del  fix\sgmlpars.dll'
end

/********************************************************************/
/* incorporate OS/2 and Device Driver fixpaks and Feature Installer */
/********************************************************************/

/* run update */
'@del result.out >nul'
'@copy basefpk.mcp basefpk.pin >nul'
'@call updcd 'burn' fix 'n.errors n.warnings trunc(time('E')/60 + n.elapsed, 2) log ver
res = linein('result.out')
rc = lineout('result.out')
parse var res n.errors n.warnings 

/* and delete temp stuff from fix */
rc = SysFileTree('fix\*', ini., 'FSO',,'**---')  
do i = 1 to ini.0
	call execute '@del 'ini.i
end

if expert = 0 then do
	say '1B'x || '[19;2HPress any key to continue or CTRL-C to abort'
	'@pause >nul'
	say '1B'x || '[19;2H                                            '
end


/************************/
/* change CID procedure */
/************************/

rc = SysFileTree(burn'\ibminst\*', ini., 'FSO',,'**---')  
call execute '@copy newinst\addons.cfg   'burn'\ibminst\tables'
call execute '@copy newinst\addon.fil    'burn'\ibminst\tables'
call execute '@copy newinst\StTimeV.Cmd  'burn'\ibminst'
call execute '@copy unzip.exe            'burn'\ibminst'
call execute '@copy 'burn'\ibminst\npconfig.exe 'burn'\ibminst\npcfg2.exe'
call execute '@copy newinst\npconfig.exe 'burn'\ibminst'
call execute '@copy vrobj.dll            'burn'\ibminst'
call execute '@copy newinst\tunecfg.cmd  'burn'\ibminst'
call execute '@copy newinst\tunecfg.cfg  'burn'\ibminst'
'@echo do not remove this file > 'burn'\ibminst\npconfig.mcp'

/* change mkrsp.cmd */
out = 'temp.out'
'@del 'out' >nul 2>>&1'
in = burn'\ibminst\mkrsp.cmd'
do while lines(in)
	l=linein(in)
	if pos('/* We completed successfully */', l) > 0 then do
		call lineout out, "/* zsolt begin */"
		call lineout out, "/* copy scripts for installing add-ons */"
		call lineout out, "'@copy 'tabledir'\ADDON.FIL 'cltdir'\ADDON.CMD'"
		call lineout out, "'@copy 'ibminstdir'\StTimeV.Cmd 'OS2_Drive'\OS2\.'"
		call lineout out, "/* zsolt end */"
		call lineout out, " "
	end
	call lineout out, l
end
call lineout in
call lineout out
'@copy 'out in' >nul 2>>&1'
'@del 'out' >nul 2>>&1'
in = burn'\ibminst\tables\lcudrvr.fil'
do while lines(in)
	l=linein(in)
	if pos('/* Cleanup srvifs */', l) > 0 then do
		call lineout out, "/* zsolt begin */"
		call lineout out, "/* Create icon for Warp 4 FI products */"
		call lineout out, "if stream(CD_Drive'\fi\fibase.rsp', 'c', 'query exists') <> '' then "
		call lineout out, "	'clifi /A:B /F:^<WP_INSTREMFOLDER^> /R:'CD_Drive'\FI\FIBASE.RSP'"
		call lineout out, " "
		call lineout out, "/* Create logdir, icons and call the addon.cmd if there is one */"
		call lineout out, "if stream(cltdir'\ADDON.CMD', 'C', 'QUERY EXISTS') \= '' then do"
		call lineout out, "	Call SysMkDir logsdir'\addon' "
		call lineout out, '	rc = SysCreateObject("WPProgram", "Selective Install^for AddOn Products", "<WP_INSTREMFOLDER>", "EXENAME="ibminstdir"\npconfig.exe;PARAMETERS=/REINSTALL;", "F")  '
		call lineout out, '	rc = SysCreateObject("WPProgram", "Selective UnInstall^for AddOn Products", "<WP_INSTREMFOLDER>", "EXENAME="ibminstdir"\npconfig.exe;PARAMETERS=/UNINSTALL;", "F") '
		call lineout out, "	'@call 'cltdir'\ADDON.CMD 'CD_Drive bootdrive percent_complete"
		call lineout out, "end"
		call lineout out, " "
	end
	call lineout out, l
end
call lineout in
call lineout out
'@copy 'out in' >nul 2>>&1'
'@del 'out' >nul 2>>&1'

/************************/
/* Add fixed getcfg.exe */
/************************/
call execute '@copy 'burn'\os2image\disk_3\getcfg.exe 'burn'\os2image\disk_3\getcfg2.exe'
call execute '@copy newinst\getcfg.exe 'burn'\os2image\disk_3\.'

/************************/
/* add UPDCD info to CD */
/************************/

call execute '@mkdir 'burn'\updcd\bin'
call execute '@copy *.cm? 'burn'\updcd\bin\.'
call execute '@copy *.exe 'burn'\updcd\bin\.'
call execute '@copy *.dll 'burn'\updcd\bin\.'
call execute '@copy *.msg 'burn'\updcd\bin\.'
call execute '@copy *.cfg 'burn'\updcd\bin\.'
call execute '@copy cd2boot   'burn'\updcd\bin\.'
call execute '@copy pmfaxlt.* 'burn'\updcd\bin\.'
call execute '@copy basefpk.* 'burn'\updcd\bin\.'
call execute '@copy mptsfpk.* 'burn'\updcd\bin\.'
call execute '@copy tcppfpk.* 'burn'\updcd\bin\.'
call execute '@copy peerfpk.* 'burn'\updcd\bin\.'
call execute '@mkdir 'burn'\updcd\doc'
call execute '@copy *.txt 'burn'\updcd\doc\.'
call execute '@copy readme.* 'burn'\updcd\doc\.'
call execute '@mkdir 'burn'\updcd\addons'
call execute '@xcopy addons\* 'burn'\updcd\addons\. /S/O/T/R/E/H'
call execute '@mkdir 'burn'\updcd\log'
rc = lineout(log, 'Info: Total number of errors  : 'n.errors)
rc = lineout(log, 'Info: Total number of warnings: 'n.warnings)
rc = lineout(log)
'@copy *.log 'burn'\updcd\log\. >nul 2>>&1'

/***********/
/* burn CD */
/***********/

if stream(cdrprg, 'c', 'query exists') <> '' then do
	say '1B'x || '[7;26H'substr('Calling script 'cdrprg' to burn CD.', 1, 50)
	say '1B'x || '[14;26H'trunc(time('E')/60 + n.elapsed, 2)
	say '1B'x || '[19;2H'
	'@call 'cdrprg n.errors n.warnings
end

/***********************/
/* clean up and report */
/***********************/
'@del result.out >nul'

/* signal fatal errors */
if n.errors > 0 then 	say '1B'x || '[19;2HThere were one or more errors detected, Check 'log'!'
else say '1B'x || '[19;2HUpdate is completed succesfully!'

'@del go4it.bsy >nul 2>>&1'
exit


/* delete directory */
DelDir: procedure expose n. log

	parse arg Directory
	DirSpec = Directory || '\*'

	/* delete subdirectories */
	rc = SysFileTree(DirSpec, Subdirs, 'DO', '*****', '-*---')
	do i = 1 to Subdirs.0
   		call DelDir Subdirs.i
	end

	/* delete files */
	rc = SysFileTree(DirSpec, Files, 'FO', '*****', '-*---')
	do i = 1 to Files.0
      call execute '@del 'Files.i
  end

	/* delete directory */
	cdir1 = directory()
	cdir2 = directory(filespec("drive", Directory))
	d = directory(Directory)
	cdir2 = directory(cdir2) 
	cdir1 = directory(cdir1) 
	if d <> '' & d <> cdir1 then call execute '@rmdir 'Directory

return

/* execute command, display it and write to log */ 
execute: procedure expose log n. iddir

	parse arg comm 
	rcode = 0

	parse upper var comm comm1 comm2 comm3 comm4 rest
	/* do not copy or pack if file does not exist */
	rc = SysFileTree(comm2, tmp., 'FSO') 
	if (comm1 <> '@COPY' & comm1 <> '@PACK2' & comm1 <> '@DEL' & comm1 <> '@XCOPY') | (stream(comm2, 'c', 'query exists') <> '') | (tmp.0 > 0) then do
		comm' >nul 2>>&1'
		if rc <> 0 then rcode = rc
	end
	else do /* log warning */
		if comm1 <> '@DEL' then do
			rc = lineout(log, 'Warning: 'comm2' does not exist!')
			n.warnings = n.warnings + 1
			call report_status
		end
		return
	end
	comm = comm ||' rc='rcode

	if comm1 = '@..\ZIP.EXE' then do	
		say '1B'x || '[8;26H'substr(space(reverse(substr(reverse(comm3), 1, 50))), 1, 50)
		say '1B'x || '[9;26H'substr(space(reverse(substr(reverse(comm4), 1, 50))), 1, 50)
		say '1B'x || '[11;26HZip         '
	end
	else if comm1 = '@MKDIR' | comm1 = '@RMDIR' | comm1 = '@CD' then do	
		nop;
	end
	else if comm1 = '@UNZIP' then do	
		say '1B'x || '[8;26H'substr(space(reverse(substr(reverse(rest), 1, 50))), 1, 50)
		say '1B'x || '[9;26H'substr(space(reverse(substr(reverse(comm3), 1, 50))), 1, 50)
		say '1B'x || '[11;26HUnzip         '
	end
	else do 
		say '1B'x || '[8;26H'substr(space(reverse(substr(reverse(comm3), 1, 50))), 1, 50)
		say '1B'x || '[9;26H'substr(space(reverse(substr(reverse(comm2), 1, 50))), 1, 50)
		say '1B'x || '[11;26H'substr(comm1, 2, 50)
	end
	say '1B'x || '[12;26H'rcode' '

	if rcode <> 0 then do
		rc = lineout(log, 'Error: 'comm)
		rc = lineout(log)
		n.errors = n.errors + 1
	end
	else do
		rc = lineout(log, 'Info: 'comm)
		rc = lineout(log)
	end

	call report_status

return rcode

/* display status */
report_status: procedure expose n. 

	say '1B'x || '[14;2HElapsed time [m]      : 'trunc(time('E')/60 + n.elapsed, 2)
	say '1B'x || '[15;2HNumber of errors      : 'n.errors
	say '1B'x || '[16;2HNumber of warnings    : 'n.warnings
	say '1B'x || '[19;2H'

return

/* copy files from sdir to tdir except ea data. sf */
CopyDir: procedure expose log n.

	parse arg sdir tdir 

	call SysFileTree sdir'\*', ini., 'FO', , '**---'
	do i=1 to ini.0
		if pos('EA DATA. SF', translate(ini.i)) = 0 then do
			'@copy 'ini.i tdir'\. >nul 2>>&1'
			rcode = rc
			if rcode <> 0 then do
				rc = lineout(log, 'Error: @copy 'ini.i tdir'\. (rc='rcode')')
				rc = lineout(log)
				n.errors = n.errors + 1
			end
			else do
				rc = lineout(log, 'Info: @copy 'ini.i tdir'\. (rc='rcode')')
				rc = lineout(log)
			end
		end
	end
	call report_status

return
