/* Burn bootable OS/2 CD-ROM using RSJ                                       */
/* Version 1.67                                                              */
/*                                                                           */
/* 11.25.2000: added code of Fred Leggett to automate the boot img creation  */
/* NOTE: Since this script uses Albert Shan's SVDISK to circumvent having    */
/*       to use a floppy for the boot img creation process, it helps to have */
/*       SVDISK registered.                                                  */
/* 12.13.2000: addes some code to align with the PM interface                */
/* 02.19.2001: fixed problem with label disk 2                               */
/* 02.27.2001: added new menupoint (6)                                       */
/* 03.04.2001: fixed problem with flag file                                  */
/* 03.19.2001: some cosmethic changes                                        */
/* 04.10.2001: solved incomplete do end bug                                  */
/* 05.10.2001: added support for VFDISK 2.0 or higher                        */
/* 05.19.2001: added more logging                                            */
/* 06.09.2001: bug fixing, small improvements                                */
/* 06.26.2001: modified by Jeffrey R Smick to support RSJ                    */
/* 08.04.2001: bug fixing by Jeffrey R Smick                                 */
/* 08.05.2001: changed name from rjsrecord.cmd to rsjrecrd.cmd               */

/* get some parameters */
parse arg n.errors n.warnings
if n.errors   = '' then n.errors   = '?'
if n.warnings = '' then n.warnings = '?'

/* global flag for attached status of cd burner */
attached = 0
/* gloabal variable to store letter to attach cd burner to (default Z:) */
CD = ""
/* global flag to register whether user has created a batch job */
automated = 0

IF RxFuncQuery('SysLoadFuncs') THEN DO
  CALL RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  CALL SysLoadFuncs
END

/* read paramaters from updcd.cfg */
cfgfile = 'updcd.cfg'
DO WHILE LINES(cfgfile)
  l = LINEIN(cfgfile)
  INTERPRET l
END
call stream cfgfile, "C", "CLOSE"


/* Find out if user has RSJ cdwriter system installed */
/* do this by checking the class list for CDWRITER class */
Call SysQueryClassList 'junk.'
i = 1
found = 0
Do while ( i <= junk.0 & \found )
   tmp = word( junk.i, 1 )
   If tmp == "CDWriter" Then Do
      /* We found the rsj CDWRITER class */
      tmp = word( junk.i, 2 )
      rsjdir = substr( tmp, 1, lastpos( "\", tmp )-1 )
      found = 1     /* exit the loop */
   End /* Do */
   i = i + 1
End /* Do */

If \found Then Do
   say "RSJ CDWriter File System could not be found in the class list."
   say "Exiting..."
   call SysSleep 3
   return
End /* Do */

/* remember we were started */
'@echo record.cmd busy > record.bsy'

/* reset */
boot = ""
key = ""
flag1 = " "
flag2 = " "
flag3 = " "
flag4 = " "
flag5 = " "
flag6 = " "
'@mode 80,25 >nul'
'@ansi on >NUL'
say '1B'x || '[36;44m'
say '1B'x || '[1m'
'@cls'

/* start menu */
begin:

text.1 = "Update CD - Imagefile & CD Burn Script"
text.1 = overlay("Update CD - Imagefile & CD Burn Script", "  :                                                                         :", trunc((74-length(text.1))/2))
text.2 = "The build procedure has ended with "n.errors" errors and "n.warnings" warnings."
text.2 = overlay("The build procedure has ended with "n.errors" errors and "n.warnings" warnings.", "  :                                                                         :", trunc((74-length(text.2))/2))
text.3 = "See ["log"] for output details."
text.3 = overlay("See ["log"] for output details.", "  :                                                                         :", trunc((74-length(text.3))/2))

DO WHILE (key \= 0)

	'@cls'
  SAY "  ---------------------------------------------------------------------------"
  SAY text.1
  SAY "  ---------------------------------------------------------------------------"
  SAY text.2
  SAY text.3
  SAY "  ---------------------------------------------------------------------------"
  SAY "  :                                                                         :"
  SAY "  :  * = Process done.                                                      :"
  SAY "  :                                                                         :"
  SAY "  :  0 - Quit.                                                              :"
  SAY "  :                                                                         :"
  SAY "  : "flag1"1 - View burn log.                                                     :"
  SAY "  : "flag2"2 - Clear burn log.                                                    :"
  SAY "  : "flag3"3 - Blank CD-RW (necessary prior to burning if using CD-RW media).     :"
  SAY "  : "flag4"4 - Create boot image files (necessary for a bootable CDROM).          :"
  SAY "  : "flag5"5 - Burn CD-R or CD-RW.                                                :"
  SAY "  :                                                                         :"
  SAY "  : "flag6"6 - Let me specify the steps that should run as a batch job.           :"
  SAY "  :                                                                         :"
  SAY "  ---------------------------------------------------------------------------"
  SAY

  key = ""
  DO WHILE (key < 0) | (key > 6)
    key = SysGetKey('NOECHO')
  END
  SAY

   SELECT

      WHEN (key == 1) THEN call view_log
      WHEN (key == 2) THEN call delete_log Q
      WHEN (key == 3) THEN call blank_cd Q
      WHEN (key == 4) THEN call make_bootimage Q
      WHEN (key == 5) THEN call burn_cd Q
      WHEN (key == 6) THEN do
         SAY "Specify the steps separated by spaces (e.g. 3 4 5 1):"

         pull response
         automated = 1
         If pos( 5, response ) > 0 & \attached Then
            If \attachDev() Then call begin

         fail = 0
         Do while length(response) > 0 & \fail

            parse var response job response
            if datatype(job) <> 'NUM' | job < 1 | job > 5 then do
               say 'Invalid response, skipping step: 'job
               pause
            end
            Else Do
               select
                  when job == 1 then call view_log
                  WHEN job == 2 THEN call delete_log Q
                  when job == 3 then call blank_cd Q
                  when job == 4 then call make_bootimage Q
                  when job == 5 then call burn_cd Q
                  otherwise nop;
               end /* select */
            end /* else */
         end /* do */
         automated = 0
         flag6 = "*"
      end /* do */
      OTHERWISE DO
         SAY "You may re-run this script by typing rsjrecord.cmd from "rootdir
         SAY
         SAY "Bye bye!"
         SAY
         /* we are ready */
         '@del 'rootdir'\record.bsy >nul 2>>&1'
      END
   END
END

EXIT

/* Common error text to print to screen. */

PrintError: procedure expose fail

   parse arg rc

  SAY "There was an error.  This process has not been completed!"
  say "The error code returned was "||rc||"."
  SAY "Please re-run this option when the problem is fixed."
  SAY "Hit any key to return to menu."
  call SysGetKey 'NOECHO'
  fail = 1

  RETURN

delete_log:
   parse arg option
   rc = 0
   Call charout , "Clearing burn log..."
   '@del '||burnlog' 1>NUL 2>&1'
   If rc == 0 Then Do
      say "Done!"
      flag2 = "*"
      call SysSleep 1
   End /* Do */
   Else if rc == 1 then say "The file could not be found."
   else say "Error! Rc reported was: "rc


   If \(option == "Q") Then Do
      call charout , "Hit any key to continue."
      call SysGetKey "NoEcho"
   End /* Do */


   return

/* procedure view log */
view_log:

      DO
        IF STREAM(burnlog,"C","query EXISTS") == "" THEN
          DO
            SAY burnlog" does not exist!"
            SAY
            SAY "A process has not yet been run or an error has occurred while trying"
            SAY "to write to "burnlog"."
            SAY
            SAY "Hit any key to return to the menu."
            SysGetKey('NOECHO')

          END
				Else Do
               flag1 = "*"
               '@start "BurnLog" /F e.exe '||burnlog
            End /* Do */
      END

return

/* procedure build boot image and burn to cd */
make_bootimage: procedure expose flag4 source CD burnlog svdc_drive rootdir automated fail rsjdir

   parse arg option

   /* make boot image files */
   IF STREAM(source'\DISK_*.IMG',"C","query EXISTS") \= "" THEN DO
      if option = 'Q' then do
         CALL SysFileDelete source'\DISK_0_1.IMG'
         CALL SysFileDelete source'\DISK_2.IMG'
      end
      else do
         call charout , "Boot image files already exist.  Overwrite?"
         IF askYN() THEN DO
            CALL SysFileDelete source'\DISK_0_1.IMG'
            CALL SysFileDelete source'\DISK_2.IMG'
         END
         else do
            flag4 = "*"
            CALL begin
         END
    	end
   END

   /* check vdisk type */
   vdisk = SysIni(, 'UPDCD', 'VDISK')
   if vdisk = "ERROR:" then do
      vdisk = "SVDISK" /* let us make SVDISK the default */
      call SysIni , 'UPDCD', 'VDISK', vdisk
   end

   '@echo Trying to insert a 2.88MB virtual disk in drive 'svdc_drive' > 'burnlog
   call charout , "Attempting to load 2.88MB virtual media..."
   rc = 0
   if vdisk = 'VFDISK' then do
   	'@vfctrl 'svdc_drive' 0     >>'burnlog' 2>>&1'
   	'@vfctrl 'svdc_drive' 4     >>'burnlog' 2>>&1'
   end
   else do
      '@SVDC /ef '    svdc_drive' >>'burnlog' 2>>&1'
      '@SVDC /r:2.88 'svdc_drive' >>'burnlog' 2>>&1'
      '@SVDC /i:2.88 'svdc_drive' >>'burnlog' 2>>&1'
   end
   if rc <> 0 then do
      say 'Error inserting a 2.88MB virtual floppy in drive 'svdc_drive
      say "Hit any key to exit this procedure."
      call SysGetKey "NoEcho"
      exit 9
   end
   say "Done!"

   call charout , "Making boot image one of two..."
   parse value SysDriveInfo( svdc_drive ) with . free .
   '@echo Detected 'free' bytes free space on drive 'svdc_drive' >> 'burnlog
   '@cd 'source'\UpdCD\bootimg\DISK_0_1.IMG >>'burnlog' 2>>&1'
   '@SYSINSTX 'svdc_drive' >>'burnlog' 2>>&1'
   '@XCOPY * 'svdc_drive' /H/O/T/S/E/R/V >>'burnlog' 2>>&1'
   '@'||rootdir||'\cdboot.exe /drive 'substr( svdc_drive, 1, 1 )' /timeout 10 >> 'burnlog' 2>>&1'
   '@cd 'source'\diskimgs >>'burnlog' 2>>&1'
   '@IMAGE 'svdc_drive' ..\DISK_0_1.IMG >>'burnlog' 2>>&1'
   say "Done!"

   '@echo Trying to insert a 1.44MB virtual disk in drive 'svdc_drive' >> 'burnlog
   call charout , "Attempting to load 1.44MB virtual media..."
   if vdisk = 'VFDISK' then do
   	'@vfctrl 'svdc_drive' 0     >>'burnlog' 2>>&1'
   	'@vfctrl 'svdc_drive' 1     >>'burnlog' 2>>&1'
   end
   else do
     '@SVDC /ef '    svdc_drive' >>'burnlog' 2>>&1'
     '@SVDC /r:1.44 'svdc_drive' >>'burnlog' 2>>&1'
     '@SVDC /i:1.44 'svdc_drive' >>'burnlog' 2>>&1'
   end
   if rc <> 0 then do
   	say 'Error inserting a 1.44MB virtual floppy in drive 'svdc_drive
      say "Hit any key to exit this procedure."
      call SysGetKey "NoEcho"
   	exit 9
   end
   say "Done!"

   call charout , "Making boot image two of two..."
   parse value SysDriveInfo( svdc_drive ) with . free .
   '@echo Detected 'free' bytes free space on drive 'svdc_drive' >> 'burnlog
   '@cd 'source'\UpdCD\bootimg\DISK_2.IMG >>'burnlog' 2>>&1'
   '@XCOPY * 'svdc_drive' /H/O/T/S/E/R/V >>'burnlog' 2>>&1'
   '@cd 'source'\diskimgs >>'burnlog' 2>>&1'
   '@label 'svdc_drive'DISK 2 >>'burnlog' 2>>&1'
   '@IMAGE 'svdc_drive' ..\DISK_2.IMG >>'burnlog' 2>>&1'
   say "Done!"

   '@echo Trying to insert a 2.88MB virtual disk in drive 'svdc_drive' >> 'burnlog
   if vdisk = 'VFDISK' then do
   	'@vfctrl 'svdc_drive' 0     >>'burnlog' 2>>&1'
   	'@vfctrl 'svdc_drive' 4     >>'burnlog' 2>>&1'
   end
   else do
     '@SVDC /ef '    svdc_drive' >>'burnlog' 2>>&1'
     '@SVDC /r:2.88 'svdc_drive' >>'burnlog' 2>>&1'
     '@SVDC /i:2.88 'svdc_drive' >>'burnlog' 2>>&1'
   end
   if rc <> 0 then do
   	say 'Error inserting a 2.88MB virtual floppy in drive 'svdc_drive
      say "Hit any key to exit this procedure."
      call SysGetKey "NoEcho"
   	exit 9
   end
   parse value SysDriveInfo(svdc_drive) with . free .
   '@echo Detected 'free' bytes free space on drive 'svdc_drive' >> 'burnlog
   flag4 = "*"

   if \automated Then Do
      call charout , "Hit any key to continue."
      call SysGetKey "NoEcho"
   End /* Do */

return

/* procedure burn cd */
burn_cd: procedure expose flag5 CD attached source burnlog automated rsjdir fail

   parse arg option
/*
isize = STREAM(tempraw,"C","query SIZE")
isizeK = isize / 1024
isizeM = isize / 1048576
itimeMIN = isize / (153600 * speed) % 60
itimeSEC = isize / (153600 * speed) // 60



   SAY "About to burn updated Warp 4 CD."
   SAY
   SAY "The imagefile size in kilobytes is:  "isizeK
   SAY "The imagefile size in megabytes is:  "FORMAT(isizeM,,2)
   SAY "(A megabyte is defined as 1,048,576 bytes {2^20 bytes}.)"
   SAY
   SAY "Burner device:  "device
   SAY "CD burner speed:  "speed "("150 * speed" kilobytes per second.)"
   SAY "This burn will complete in approximately "itimeMIN":"FORMAT(itimeSEC,,2)" minutes."
   SAY "The time now is "TIME('C')
*/

   If \attached Then
      If \attachDev() Then Do
         fail = 1
         return
      End /* Do */

   rc = 0

   call charout ,   "Burning updated installation files to cd..."
/*   cmd = rsjdir||'\cdcopy '||source||'\DISK_2.IMG '||CD||'\DISK_2.IMG' */
   cmd = rsjdir||'\cdcopy '||source||'\*.* '||CD||'\ /s'
   '@echo '||cmd||' >> '||burnlog
   '@'||cmd||' >>'||burnlog||' 2>>&1'

   IF (rc == 0) Then Do
      say "Done!"
      IF STREAM(source'\DISK_0_1.IMG',"C","query EXISTS") \= "" Then Do
         call charout , "Found bootable image files. Burning boot sector data to cd..."
         /* now make image file bootable */
         cmd = rsjdir||'\cdboot.exe '||CD||'\DISK_0_1.IMG -s '||source||'\DISK_0_1.IMG'
         '@echo '||cmd||' >> '||burnlog
         '@'||cmd||' >>'||burnlog||' 2>>&1'
         IF (rc \== 0) Then Do
            say "Error!"
            say "Boot sector data was not transferred to cd."
            CALL PrintError rc
         End /* Do */
         Else Do
            say "Done!"
            If detach() Then flag5 = "*"
         End /* Do */
      end /* if */
      Else Do
         If detach() Then flag5 = "*"
      End /* Do */
   End /* Do */
   Else Do
      say  "Error!"
      CALL PrintError rc
      fail = 1
   End /* Do */

   If \automated Then Do
      call charout , "Hit any key to continue."
      call SysGetKey "NoEcho"
   End /* Do */

return

/* procedure blank CDRW */
blank_cd: procedure expose burnlog attached CD flag3 automated fail rsjdir

   parse arg option

   If \attached Then
      If \attachDev() Then Do
         fail = 1 /* failure */
         return
      End /* Do */

   /* erase CDRW */
   cmd = rsjdir'\cdwfmt '||CD||' /ERASE'
   call charout , "Blanking cd..."
   rc = 0
   '@echo '||cmd||' >> '||burnlog
   '@'||cmd||' >>'||burnlog||' 2>>&1'

   IF (rc \== 0) Then Do
      say "Error!"
      CALL PrintError
   End /* Do */
   Else Do
      say "Done!"
      if detach() then flag3 = "*"
   End /* Do */
   if option \== 'Q' then do
      SAY "Completed with return code "rc
      SAY "Hit any key to return to menu."
      call SysGetKey 'NOECHO'
   end

   If \automated Then Do
      call charout , "Hit any key to continue."
      call SysGetKey "NoEcho"
   End /* Do */

return

detach: procedure expose CD burnlog attached rsjdir
   success = 1
   cmd = rsjdir'\cdattach '||CD||' -s'
   call charout , "Finalizing session..."
   '@echo '||cmd||' >> '||burnlog
   '@'||cmd||' >>'||burnlog||' 2>>&1'

   IF (rc \== 0) Then Do
      say "Error!"
      success = 0
      CALL PrintError rc
   End /* Do */
   Else Do
      say "Done!"
      attached = 0
   End /* Do */

return success


attachDev: procedure expose burnlog CD attached rsjdir

   If CD == "" Then Do
      tmp =  SysIni( "USER", "CDWFS", "DefaultDriveLetter" )
      If tmp == "" | tmp == "ERROR:" Then Do
         tmp = "Z:"
      End /* Do */
      else tmp = strip( tmp, 'T', '00'x )
      call charout , "Type the drive letter you wish to attach to your burner (default "||tmp||")."
      pull junk
      If junk \== "" Then CD = substr( junk, 1, 1 )||":"
      ELSE CD = tmp
   End /* Do */

   cmd = rsjdir'\cdattach.exe '||CD

   Do forever
      rc = 0
      call charout , "Attaching cd..."
      '@echo '||cmd||' >> '||burnlog
      '@'||cmd||' >>'||burnlog||' 2>>&1'
      Select
         When rc == 0 Then Do
            attached = 1
            say "Done!"
            leave
         End /* Do */
         When rc == 21 Then Do
            say "Error!"
            say "Could not attach media. Check to make sure media is in drive."
            call charout , "Would you like me to try again?"
            If \askYN() Then Do
               leave
            End /* Do */
         End /* Do */
         When rc == 85 Then Do
            say "Error!"
            call charout , "Warning! Media is already attached. Continue anyways?"
            If askYN() Then Do
               attached = 1
            End /* Do */
            leave
         End /* Do */
         Otherwise do
            say "Error!"
            say "An error has occurred. The device could not be attached."
            say "This procedure will now end. Hit any key to continue."
            call SysGetKey "NOECHO"
            leave
         end
      End  /* Select */

   End /* Do forever */

return attached


askYN: procedure
   call charout  , " [y/n]"
   bool = 0
   resp = ""
   Do while ((resp <> "Y") & (resp <> "N"))
      resp = translate( SysGetKey( NOECHO ))
      If resp == "Y" Then bool = 1
   End /* Do */
   say resp
   return bool


